@@ -16,13 +16,13 @@ typedef struct Buffer_t {
 	size_t size;     	/* size of read/write buffer */
 	int dirty;	       	/* is the buffer dirty? */
 
-	int sectorSize;		/* sector size: all operations happen
+	size_t sectorSize;	/* sector size: all operations happen
 				 * in multiples of this */
-	int cylinderSize;	/* cylinder size: preferred alignemnt,
+	size_t cylinderSize;	/* cylinder size: preferred alignemnt,
 				 * but for efficiency, less data may be read */
 	int ever_dirty;	       	/* was the buffer ever dirty? */
-	int dirty_pos;
-	int dirty_end;
+	size_t dirty_pos;
+	size_t dirty_end;
 	mt_off_t current;		/* first sector in buffer */
 	size_t cur_size;		/* the current size */
 	char *buf;		/* disk read/write buffer */
@@ -43,7 +43,7 @@ static int _buf_flush(Buffer_t *Buffer)
 		fprintf(stderr,"Should not happen\n");
 		return -1;
 	}
-#if DEBUG
+#ifdef DEBUG
 	fprintf(stderr, "write %08x -- %02x %08x %08x\n",
 		Buffer,
 		(unsigned char) Buffer->buf[0],
@@ -55,7 +55,7 @@ static int _buf_flush(Buffer_t *Buffer)
 			  Buffer->buf + Buffer->dirty_pos,
 			  Buffer->current + Buffer->dirty_pos,
 			  Buffer->dirty_end - Buffer->dirty_pos);
-	if(ret != Buffer->dirty_end - Buffer->dirty_pos) {
+	if(ret != (signed int) (Buffer->dirty_end - Buffer->dirty_pos)) {
 		if(ret < 0)
 			perror("buffer_flush: write");
 		else
@@ -175,14 +175,14 @@ static int buf_write(Stream_t *Stream, char *buf, mt_off_t start, size_t len)
 {
 	char *disk_ptr;
 	DeclareThis(Buffer_t);	
-	int offset;
+	size_t offset;
 
 	if(!len)
 		return 0;
 
 	This->ever_dirty = 1;
 
-#if DEBUG
+#ifdef DEBUG
 	fprintf(stderr, "buf write %x   %02x %08x %08x -- %08x %08x -- %08x\n", 
 		Stream, (unsigned char) This->buf[0],
 		start, len, This->current, This->cur_size, This->size);
@@ -193,7 +193,7 @@ static int buf_write(Stream_t *Stream, char *buf, mt_off_t start, size_t len)
 #endif
 	switch(isInBuffer(This, start, &len)) {
 		case OUTSIDE:
-#if DEBUG
+#ifdef DEBUG
 			fprintf(stderr, "outside\n");
 #endif
 			if(start % This->cylinderSize || 
@@ -227,7 +227,7 @@ static int buf_write(Stream_t *Stream, char *buf, mt_off_t start, size_t len)
 			}
 			/* FALL THROUGH */
 		case APPEND:
-#if DEBUG
+#ifdef DEBUG
 			fprintf(stderr, "append\n");
 #endif
 			len = ROUND_DOWN(len, This->sectorSize);
@@ -240,7 +240,7 @@ static int buf_write(Stream_t *Stream, char *buf, mt_off_t start, size_t len)
 			break;
 		case INSIDE:
 			/* nothing to do */
-#if DEBUG
+#ifdef DEBUG
 			fprintf(stderr, "inside\n");
 #endif
 			offset = OFFSET;
@@ -249,7 +249,7 @@ static int buf_write(Stream_t *Stream, char *buf, mt_off_t start, size_t len)
 		case ERROR:
 			return -1;
 		default:
-#if DEBUG
+#ifdef DEBUG
 			fprintf(stderr, "Should not happen\n");
 #endif
 			exit(1);
