@@ -13,13 +13,14 @@
 static char buffer[MAX_LINE_LEN+1]; /* buffer for the whole line */
 static char *pos; /* position in line */
 static char *token; /* last scanned token */
-static int token_length; /* length of the token */
+static size_t token_length; /* length of the token */
 static FILE *fp; /* file pointer for configuration file */
 static int linenumber; /* current line number. Only used for printing
 						* error messages */
 static int lastTokenLinenumber; /* line numnber for last token */
-static const char *filename; /* current file name. Only used for printing
-			      * error messages */
+static const char *filename=NULL; /* current file name. Used for printing
+				   * error messages, and for storing in
+				   * the device definition (mtoolstest) */
 static int file_nr=0;
 
 
@@ -33,6 +34,8 @@ static int nr_dev; /* number of devices that the current table can hold */
 struct device *devices; /* the device table */
 static int token_nr; /* number of tokens in line */
 
+static char default_drive='\0'; /* default drive */
+
 /* "environment" variables */
 unsigned int mtools_skip_check=0;
 unsigned int mtools_fat_compatibility=0;
@@ -43,7 +46,7 @@ unsigned int mtools_no_vfat=0;
 unsigned int mtools_numeric_tail=1;
 unsigned int mtools_dotted_dir=0;
 unsigned int mtools_twenty_four_hour_clock=1;
-char *mtools_date_string="yyyy-mm-dd";
+const char *mtools_date_string="yyyy-mm-dd";
 char *country_string=0;
 
 typedef struct switches_l {
@@ -56,7 +59,7 @@ typedef struct switches_l {
     } type;
 } switches_t;
 
-static switches_t switches[] = {
+static switches_t global_switches[] = {
     { "MTOOLS_LOWER_CASE", (caddr_t) & mtools_ignore_short_case, T_UINT },
     { "MTOOLS_FAT_COMPATIBILITY", (caddr_t) & mtools_fat_compatibility, T_UINT },
     { "MTOOLS_SKIP_CHECK", (caddr_t) & mtools_skip_check, T_UINT },
@@ -151,6 +154,23 @@ static switches_t dswitches[]= {
     { "BLOCKSIZE", OFFS(blocksize), T_UINT }
 };
 
+static void maintain_default_drive(char drive)
+{
+    if(default_drive == ':')
+	return; /* we have an image */
+    if(default_drive == '\0' ||
+       default_drive > drive)
+	default_drive = drive;    
+}
+
+char get_default_drive(void)
+{
+    if(default_drive != '\0')
+	return default_drive;
+    else
+	return 'A';
+}
+
 static void syntax(const char *msg, int thisLine)
 {
     char drive='\0';
@@ -168,24 +188,24 @@ static void syntax(const char *msg, int thisLine)
 static void get_env_conf(void)
 {
     char *s;
-    int i;
+    unsigned int i;
 
-    for(i=0; i< sizeof(switches) / sizeof(*switches); i++) {
-	s = getenv(switches[i].name);
+    for(i=0; i< sizeof(global_switches) / sizeof(*global_switches); i++) {
+	s = getenv(global_switches[i].name);
 	if(s) {
-	    if(switches[i].type == T_INT)
-		* ((int *)switches[i].address) = (int) strtol(s,0,0);
-	    if(switches[i].type == T_UINT)
-		* ((int *)switches[i].address) = (unsigned int) strtoul(s,0,0);
-	    else if (switches[i].type == T_STRING)
-		* ((char **)switches[i].address) = s;
+	    if(global_switches[i].type == T_INT)
+		* ((int *)global_switches[i].address) = (int) strtol(s,0,0);
+	    if(global_switches[i].type == T_UINT)
+		* ((int *)global_switches[i].address) = (unsigned int) strtoul(s,0,0);
+	    else if (global_switches[i].type == T_STRING)
+		* ((char **)global_switches[i].address) = s;
 	}
     }
 }
 
 static int mtools_getline(void)
 {
-    if(!fgets(buffer, MAX_LINE_LEN, fp))
+    if(!fp || !fgets(buffer, MAX_LINE_LEN, fp))
 	return -1;
     linenumber++;
     pos = buffer;
@@ -407,7 +427,7 @@ static int set_var(struct switches_l *switches, int nr,
 
 static int set_openflags(struct device *dev)
 {
-    int i;
+    unsigned int i;
 
     for(i=0; i < sizeof(openflags) / sizeof(*openflags); i++) {
 	if(match_token(openflags[i].name)) {
@@ -420,7 +440,7 @@ static int set_openflags(struct device *dev)
 
 static int set_misc_flags(struct device *dev)
 {
-    int i;
+    unsigned int i;
 
     for(i=0; i < sizeof(misc_flags) / sizeof(*misc_flags); i++) {
 	if(match_token(misc_flags[i].name)) {
@@ -446,7 +466,7 @@ static int set_misc_flags(struct device *dev)
 
 static int set_def_format(struct device *dev)
 {
-    int i;
+    unsigned int i;
 
     for(i=0; i < sizeof(default_formats)/sizeof(*default_formats); i++) {
 	if(match_token(default_formats[i].name)) {
@@ -491,15 +511,31 @@ static void get_toupper(void)
 	mstoupper[i] = get_number();
 }
 
+static int parse_one(int privilege);
+
 void set_cmd_line_image(char *img, int flags) {
   prepend();
   devices[cur_dev].drive = ':';
+  default_drive = ':';
   devices[cur_dev].name = strdup(img);
   devices[cur_dev].fat_bits = 0;
   devices[cur_dev].tracks = 0;
   devices[cur_dev].heads = 0;
   devices[cur_dev].sectors = 0;
   devices[cur_dev].offset = 0;
+  if (strchr(devices[cur_dev].name, '|')) {
+    char *pipechar = strchr(devices[cur_dev].name, '|');
+    *pipechar = 0;
+    strncpy(buffer, pipechar+1, MAX_LINE_LEN);
+    buffer[MAX_LINE_LEN] = '\0';
+    fp = NULL;
+    filename = "{command line}";
+    linenumber = 0;
+    lastTokenLinenumber = 0;
+    pos = buffer;
+    token = 0;
+    while (parse_one(0));
+  }
 }
 
 static void parse_old_device_line(char drive)
@@ -548,6 +584,7 @@ static void parse_old_device_line(char drive)
     }
 	
     devices[cur_dev].drive = toupper(devices[cur_dev].drive);
+    maintain_default_drive(devices[cur_dev].drive);
     if (!(devices[cur_dev].name = strdup(name))) {
 	printOom();
 	exit(1);
@@ -588,6 +625,7 @@ static int parse_one(int privilege)
 	trusted = privilege;
 	flag_mask = 0;
 	devices[cur_dev].drive = toupper(token[0]);
+	maintain_default_drive(devices[cur_dev].drive);
 	expect_char(':');
 	return 1;
     }
@@ -621,19 +659,29 @@ static int parse_one(int privilege)
 	 set_openflags(&devices[cur_dev]) &&
 	 set_misc_flags(&devices[cur_dev]) &&
 	 set_def_format(&devices[cur_dev]))) &&
-       set_var(switches,
-	       sizeof(switches)/sizeof(*switches), 0))
+       set_var(global_switches,
+	       sizeof(global_switches)/sizeof(*global_switches), 0))
 	syntax("unrecognized keyword", 1);
     return 1;
 }
 
 static int parse(const char *name, int privilege)
 {
+    if(fp) {
+	fprintf(stderr, "File descriptor already set (%p)!\n", fp);
+	exit(1);
+    }
     fp = fopen(name, "r");
     if(!fp)
 	return 0;
     file_nr++;
-    filename = strdup(name);
+    filename = name; /* no strdup needed: although lifetime of variable
+			exceeds this function (due to dev->cfg_filename),
+			we know that the name is always either
+			1. a constant
+			2. a statically allocate buffer
+			3. an environment variable that stays unchanged
+		     */
     linenumber = 0;
     lastTokenLinenumber = 0;
     pos = 0;
@@ -643,6 +691,8 @@ static int parse(const char *name, int privilege)
     while(parse_one(privilege));
     finish_drive_clause();
     fclose(fp);
+    filename = NULL;
+    fp = NULL;
     return 1;
 }
 
@@ -650,7 +700,7 @@ void read_config(void)
 {
     char *homedir;
     char *envConfFile;
-    char conf_file[MAXPATHLEN+sizeof(CFG_FILE1)];
+    static char conf_file[MAXPATHLEN+sizeof(CFG_FILE1)];
 
 	
     /* copy compiled-in devices */
