@@ -73,6 +73,14 @@ static int compare_geom(struct device *dev, struct device *orig_dev)
 #define devices const_devices
 
 
+#ifdef __CYGWIN__
+#define predefined_devices
+struct device devices[] = {
+   {"\\\\\\\\.\\\\A:", 'A', GENFD },
+};
+#endif /* CYGWIN */
+
+
 #ifdef OS_aux
 #define predefined_devices
 struct device devices[] = {
@@ -709,8 +717,8 @@ int analyze_one_reply(RawRequest_t *raw_cmd, int *bytes, int do_print)
 
 #define predefined_devices
 struct device devices[] = {
-	{"/dev/fd0", 'A', 0, O_EXCL, 80,2, 18,0, MDEF_ARG},
-	{"/dev/fd1", 'B', 0, O_EXCL, 0,0, 0,0, FDEF_ARG},
+	{"/dev/fd0", 'A', 0, 0, 80,2, 18,0, MDEF_ARG},
+	{"/dev/fd1", 'B', 0, 0, 0,0, 0,0, FDEF_ARG},
 	/* we assume that the Zip or Jaz drive is the second on the SCSI bus */
 	{"/dev/sdb4",'J', GENHD },
 	{"/dev/sdb4",'Z', GENHD },
@@ -799,7 +807,7 @@ struct device devices[] = {
 #endif /* __FreeBSD__ */
  
 /*** /jes -- for ALR 486 DX4/100 ***/
-#if defined(OS_netbsd)
+#if defined(OS_netbsd) || defined(OS_netbsdelf)
 #define predefined_devices
 struct device devices[] = {
 	{"/dev/rfd0a", 'A', FHD312},
@@ -932,6 +940,11 @@ struct device devices[] = {
 #endif
 #endif /* sysv4 */
 
+#ifdef OS_mingw32msvc
+#define predefined_devices
+struct device devices[] = {};
+#endif
+
 #ifdef INIT_GENERIC
 
 #ifndef USE_2M
