@@ -93,25 +93,25 @@ static int recalcPreallocSize(File_t *This)
 }
 
 static int _loopDetect(unsigned int *oldrel, unsigned int rel, 
-					   unsigned int *oldabs, unsigned int abs)
+		       unsigned int *oldabs, unsigned int absol)
 {
-	if(*oldrel && rel > *oldrel && abs == *oldabs) {
+	if(*oldrel && rel > *oldrel && absol == *oldabs) {
 		fprintf(stderr, "loop detected! oldrel=%d newrel=%d abs=%d\n",
-				*oldrel, rel, abs);
+				*oldrel, rel, absol);
 		return -1;
 	}
 
 	if(rel >= 2 * *oldrel + 1) {
 		*oldrel = rel;
-		*oldabs = abs;
+		*oldabs = absol;
 	}
 	return 0;
 }
 
 
-static int loopDetect(File_t *This, unsigned int rel, unsigned int abs)
+static int loopDetect(File_t *This, unsigned int rel, unsigned int absol)
 {
-	return _loopDetect(&This->loopDetectRel, rel, &This->loopDetectAbs, abs);
+	return _loopDetect(&This->loopDetectRel, rel, &This->loopDetectAbs, absol);
 }
 
 static unsigned int _countBlocks(Fs_t *This, unsigned int block)
@@ -199,7 +199,7 @@ static int normal_map(File_t *This, off_t where, size_t *len, int mode,
 						   mt_off_t *res)
 {
 	int offset;
-	off_t end;
+	size_t end;
 	int NrClu; /* number of clusters to read */
 	unsigned int RelCluNr;
 	unsigned int CurCluNr;
@@ -288,7 +288,9 @@ static int normal_map(File_t *This, off_t where, size_t *len, int mode,
 	maximize(*len, (1 + CurCluNr - RelCluNr) * clus_size - offset);
 	
 	end = where + *len;
-	if(batchmode && mode == MT_WRITE && end >= This->FileSize) {
+	if(batchmode && 
+	   mode == MT_WRITE && 
+	   end >= This->FileSize) {
 		*len += ROUND_UP(end, clus_size) - end;
 	}
 
@@ -310,7 +312,7 @@ static int root_map(File_t *This, off_t where, size_t *len, int mode,
 {
 	Fs_t *Fs = This->Fs;
 
-	if(Fs->dir_len * Fs->sector_size < where) {
+	if(Fs->dir_len * Fs->sector_size < (size_t) where) {
 		*len = 0;
 		errno = ENOSPC;
 		return -2;
@@ -359,9 +361,10 @@ static int write_file(Stream_t *Stream, char *buf, mt_off_t iwhere, size_t len)
 		ret = force_write(Disk, buf, pos, len);
 	else
 		ret = WRITES(Disk, buf, pos, len);
-	if(ret > requestedLen)
+	if(ret > (signed int) requestedLen)
 		ret = requestedLen;
-	if (ret > 0 && where + ret > This->FileSize )
+	if (ret > 0 && 
+	    where + ret > (off_t) This->FileSize )
 		This->FileSize = where + ret;
 	recalcPreallocSize(This);
 	return ret;
@@ -405,7 +408,7 @@ static __inline__ time_t conv_stamp(struct directory *dir)
 		tzone = tz.tz_minuteswest * 60L;
 	}
 #else
-#ifdef HAVE_TZSET
+#if defined HAVE_TZSET && !defined OS_mingw32msvc
 	{
 #if !defined OS_ultrix && !defined OS_cygwin
 		/* Ultrix defines this to be a different type */
