@@ -15,7 +15,7 @@
 
 #define DEBUG 0
 
-#include "config.h"
+#include "sysincludes.h"
 
 #ifdef USE_FLOPPYD
 
@@ -38,8 +38,6 @@
 #define SOMAXCONN 5
 #endif
 
-extern int errno;
-
 /*
    To compile:
 
@@ -96,12 +94,9 @@ typedef unsigned long Dword;
 #define BUFFERED_IO_SIZE         16348
 
 
-void serve_client(int sock, char* device_name);
+void serve_client(int sock, char **device_name, int n_dev);
 
 
-
-#define New(type) ((type*)(malloc(sizeof(type))))
-
 #ifdef USE_FLOPPYD_BUFFERED_IO
 typedef struct io_buffer {
 	Byte out_buffer[BUFFERED_IO_SIZE];
@@ -114,7 +109,7 @@ typedef struct io_buffer {
 	int handle;
 } *io_buffer;
 
-io_buffer new_io_buffer (int _handle) {
+static io_buffer new_io_buffer (int _handle) {
 	io_buffer buffer;
 
 	buffer = New(struct io_buffer);
@@ -133,13 +128,13 @@ static void flush(io_buffer buffer) {
 	}
 }
 
-void free_io_buffer(io_buffer buffer) {
+static void free_io_buffer(io_buffer buffer) {
 	flush(buffer);
 	free(buffer);
 }
 
 
-size_t buf_read (io_buffer buf, Byte* buffer, size_t nbytes) {
+static size_t buf_read (io_buffer buf, Byte* buffer, size_t nbytes) {
 	size_t rval;
 	
 	if (nbytes <= buf->in_valid) {
@@ -181,7 +176,7 @@ size_t buf_read (io_buffer buf, Byte* buffer, size_t nbytes) {
 	return rval;
 }
 
-size_t buf_write(io_buffer buf, void* buffer, size_t nbytes) {
+static size_t buf_write(io_buffer buf, void* buffer, size_t nbytes) {
 	if (buf->out_valid + nbytes > BUFFERED_IO_SIZE) {
 		flush(buf);
 		return write(buf->handle, buffer, nbytes);
@@ -224,25 +219,10 @@ typedef struct Packet {
 	Dword alloc_size;
 } *Packet;
 
-Dword byte2dword(Byte* val) {
-	Dword l;
-	l = (val[0] << 24) + (val[1] << 16) + (val[2] << 8) + val[3];
-	
-#if DEBUG
-	fprintf(stderr, "byte2dword(): %ld, 0x%x\n", l, (unsigned int) l);
-#endif		
-	
-	return l;
-}	
+#include "byte_dword.h"
 
-void dword2byte(Dword parm, Byte* rval) {
-	rval[0] = (parm >> 24) & 0xff;
-	rval[1] = (parm >> 16) & 0xff;
-	rval[2] = (parm >> 8)  & 0xff;
-	rval[3] = parm         & 0xff;
-}
-
-Dword read_dword(io_buffer fp) {
+static Dword read_dword(io_buffer fp)
+{
 	Byte val[4];
 	if (buf_read(fp, val, 4) < 4) {
 		return 0xffffffff;
@@ -251,7 +231,8 @@ Dword read_dword(io_buffer fp) {
 	return byte2dword(val);
 }
 
-void write_dword(io_buffer fp, Dword parm) {
+static void write_dword(io_buffer fp, Dword parm)
+{
 	Byte val[4];
 	
 	dword2byte(parm, val);
@@ -260,7 +241,8 @@ void write_dword(io_buffer fp, Dword parm) {
 }
 
 
-Packet newPacket() {
+static Packet newPacket(void)
+{
 	Packet packet;
 
 	packet = New(struct Packet);
@@ -270,13 +252,15 @@ Packet newPacket() {
 }
 
 
-void destroyPacket(Packet packet) {
+static void destroyPacket(Packet packet)
+{
 	if(packet->data)
 		free(packet->data);
 	free(packet);
 }
 
-void kill_packet(Packet packet) {
+static void kill_packet(Packet packet)
+{
 	if(packet->data)
 		free(packet->data);
 	packet->data = NULL;
@@ -284,7 +268,8 @@ void kill_packet(Packet packet) {
 	packet->alloc_size = 0;
 }
 
-void make_new(Packet packet, unsigned long l) {
+static void make_new(Packet packet, unsigned long l)
+{
 	if (l < packet->alloc_size) {
 		packet->len = l;
 		return;
@@ -295,7 +280,8 @@ void make_new(Packet packet, unsigned long l) {
 	memset(packet->data, 0, l);
 }
 
-char send_packet(Packet packet, io_buffer fp) {
+static char send_packet(Packet packet, io_buffer fp)
+{
 	if (packet->data) {
 		write_dword(fp, packet->len);
 		buf_write(fp, packet->data, packet->len);
@@ -316,7 +302,8 @@ char send_packet(Packet packet, io_buffer fp) {
 	return (packet->data != NULL);
 }
 
-char recv_packet(Packet packet, io_buffer fp, Dword maxlength) {
+static char recv_packet(Packet packet, io_buffer fp, Dword maxlength)
+{
 	int start;
 	int l;
 	Dword length = read_dword(fp);
@@ -351,32 +338,28 @@ char recv_packet(Packet packet, io_buffer fp, Dword maxlength) {
 	return 1;
 }
 
-void read_packet(Packet packet, int fd, int length) {
+static void read_packet(Packet packet, int fd, int length) {
 	make_new(packet, length);
 	packet->len = read(fd, packet->data, packet->len);
 }
 
-int write_packet(Packet packet, int fd) {
+static int write_packet(Packet packet, int fd) {
 	return (write(fd, packet->data, packet->len));
 }
 
-void put_dword(Packet packet, int index, Dword val) {
-	dword2byte(val, packet->data+index);
+static void put_dword(Packet packet, int my_index, Dword val) {
+	dword2byte(val, packet->data+my_index);
 }
 
-Dword get_dword(Packet packet, int index) {
-	return byte2dword(packet->data+index);
+static Dword get_dword(Packet packet, int my_index) {
+	return byte2dword(packet->data+my_index);
 }	
 
-char is_valid(Packet packet) {
-	return (packet->data != NULL);
-}
-
-Dword get_length(Packet packet) {
+static Dword get_length(Packet packet) {
 	return packet->len;
 }
 
-int eat(char **ptr, int *len, unsigned char c) {
+static int eat(char **ptr, int *len, unsigned char c) {
     /* remove length + size code + terminating 0 */
     if (*len < c + 3)
 	return -1;
@@ -385,8 +368,11 @@ int eat(char **ptr, int *len, unsigned char c) {
     return 0;
 }
 
+static const char *dispName;
 
-char do_auth(io_buffer sock, int *version) 
+static char XAUTHORITY[]="XAUTHORITY";
+
+static char do_auth(io_buffer sock, int *version) 
 {
 	int fd;
 	Display* displ;
@@ -395,6 +381,7 @@ char do_auth(io_buffer sock, int *version)
 	char *ptr;
 	int len;
 
+	char authFile[41]="/tmp/floppyd.XXXXXX";
 	char template[4096];
 
 	Packet reply = newPacket();
@@ -441,12 +428,10 @@ char do_auth(io_buffer sock, int *version)
 		return 0;
 	}
 
-	/* CPhipps 2000/02/11 - Do the open and test in one call, to avoid
-	 * race condition. Open with safe permissions. */
-	fd = open(XauFileName(), O_WRONLY | O_CREAT | O_EXCL, 0600);
-	
-	/* Locked! */
-	if (fd == -1) {
+	umask(077);
+	fd = mkstemp(authFile);
+	if(fd == -1) {
+		/* Different error than file exists */
 		put_dword(reply, 0, AUTH_DEVLOCKED);
 		send_packet(reply, sock);
 		close(fd);
@@ -454,6 +439,17 @@ char do_auth(io_buffer sock, int *version)
 		destroyPacket(mit_cookie);
 		return 0;
 	}
+#ifdef HAVE_SETENV
+	setenv(XAUTHORITY, authFile, 1);
+#else
+	{
+	  char *buffer=malloc(strlen(XAUTHORITY)+strlen(authFile)+2);
+	  strcpy(buffer, XAUTHORITY);
+	  strcat(buffer, "=");
+	  strcat(buffer, authFile);
+	  putenv(buffer);
+	}
+#endif
 
 	ptr = template;
 	ptr[4095] = 0;
@@ -489,7 +485,7 @@ char do_auth(io_buffer sock, int *version)
 
 	destroyPacket(mit_cookie);
 
-	displ = XOpenDisplay(":0");
+	displ = XOpenDisplay(dispName);
 	if (!displ) {
 		unlink(XauFileName());
 		put_dword(reply, 0, AUTH_AUTHFAILED);
@@ -502,6 +498,7 @@ char do_auth(io_buffer sock, int *version)
 	put_dword(reply, 0, AUTH_SUCCESS);
 	send_packet(reply, sock);
 	destroyPacket(reply);
+	unlink(XauFileName());
 	return 1;
 }
 
@@ -694,7 +691,7 @@ static void alarm_signal(int a)
 /*
  * This is the main loop when running as a server.
  */
-static void server_main_loop(int sock, char* device_name)
+static void server_main_loop(int sock, char **device_name, int n_dev)
 {
 	struct sockaddr_in	addr;
 	unsigned int		len;
@@ -728,7 +725,7 @@ static void server_main_loop(int sock, char* device_name)
 				 * Start the proxy work in the new socket.
 				 */
 #endif
-				serve_client(new_sock,device_name);
+				serve_client(new_sock,device_name, n_dev);
 				exit(0);
 #if DEBUG == 0
 		}
@@ -744,7 +741,7 @@ static void server_main_loop(int sock, char* device_name)
 /*
  * Print some basic help information.
  */
-static void usage(char *prog, char *opt)
+static void usage(char *prog, const char *opt)
 {
 	if (opt)
 		{
@@ -761,20 +758,11 @@ static void usage(char *prog, char *opt)
 }
 
 
-char *makeDisplayName(char *base, int port)
+static char *makeDisplayName(int dispNr)
 {
-	char *result;
-	int displaynr = (port - FLOPPYD_DEFAULT_PORT) % 10;
-	int len=strlen(base);
-
-	result = malloc(len+5);
-	strcpy(result, base);
-	result[len++] = ':';
-	result[len++] = '0' + displaynr;
-	result[len++] = '.';
-	result[len++] = '0';
-	result[len]='\0';
-	return result;
+	char result[80];
+	sprintf(result, ":%d.0", dispNr);
+	return strdup(result);
 }
 
 int main (int argc, char** argv) 
@@ -783,25 +771,23 @@ int main (int argc, char** argv)
 	int			arg;
 	int			run_as_server = 0;
 	IPaddr_t		bind_ip = INADDR_ANY;
-	short			bind_port = FLOPPYD_DEFAULT_PORT;
+	unsigned short		bind_port = 0;
 	uid_t			run_uid = 65535;
 	gid_t			run_gid = 65535;
 	char*			username = strdup("nobody");
 	int			sock;
 	int			port_is_supplied = 0;
-	int			no_local = 0;
 
 	char *server_hostname=NULL;
-	char* device_name = NULL; 
-	char *authFile=0;
-	char *p, *q;
-	const char *authKey = "XAUTHORITY";
+	char **device_name = NULL; 
+	char *floppy0 = "/dev/fd0";
+	int n_dev;
 
 
 	/*
 	 * Parse the command line arguments.
 	 */
-	while ((arg = getopt(argc, argv, "lds:r:b:")) != EOF)
+	while ((arg = getopt(argc, argv, "ds:r:b:x:")) != EOF)
 		{
 			switch (arg)
 				{
@@ -825,9 +811,8 @@ int main (int argc, char** argv)
 						bind_ip = getipaddress(optarg);
 						server_hostname=optarg;
 						break;
-
-					case 'l':
-						no_local = 1;
+					case 'x':
+						dispName = strdup(optarg);
 						break;
 
 					case '?':
@@ -837,11 +822,25 @@ int main (int argc, char** argv)
 		}
 
 	if(optind < argc) {
-		device_name = argv[optind++];
+		device_name = argv + optind;
+		n_dev = argc - optind;
+	} else {
+		device_name = &floppy0;
+		n_dev = 1;
 	}
 
-	if(optind < argc) {
-		fprintf(stderr, "Ignoring extra argument %s\n", argv[optind]);
+	if(dispName == NULL)
+		dispName = getenv("DISPLAY");
+	if(dispName==NULL && bind_port != 0)
+		dispName=makeDisplayName((unsigned short)(bind_port - 5703));
+	if(dispName==NULL)		
+		dispName=":0";
+
+	if(bind_port == 0) {
+		char *p = strchr(dispName,':');
+		bind_port = FLOPPYD_DEFAULT_PORT;
+		if(p != NULL)
+			bind_port += atoi(p+1);
 	}
 
 	if(!run_as_server) {
@@ -865,112 +864,84 @@ int main (int argc, char** argv)
 	if (run_as_server && (bind_ip == INADDR_NONE)) {
 		usage(argv[0], "The server ipaddr is invalid.");
 	}
-	if (run_as_server && (bind_port == -1))	{
+	if (run_as_server && (bind_port == 0))	{
 		usage(argv[0], "No server port was specified (or it was invalid).");
 	}
-	
-	if(!device_name) {
-		device_name = "/dev/fd0";
-	}
 
-	if (!device_name || !*device_name) {
-		usage(argv[0], "No device name specified.");
-	}
-
-	authFile = malloc(strlen(authKey) + 1+ strlen(device_name)*2+7);
-	strcpy(authFile, authKey);
-	strcat(authFile, "=/tmp/");
-	for(p=device_name, q = strlen(authKey) + authFile+6; *p; p++) {
-		switch(*p) {
-			case '/':
-				*q++ = '-';
-				*q++ = '+';
-				break;
-			case '-':
-				*q++ = '-';
-				*q++ = '-';
-				break;
-			default:
-				*q++ = *p;
-		}
-	}
-	*q = *p;
-	putenv(authFile);
 
 	/*
 	 * See if we should run as a server.
 	 */
-	if (run_as_server)
-		{
-			/*
-			 * Start by binding to the port, the child inherits this socket.
-			 */
-			sock = bind_to_port(bind_ip, bind_port);
-
-			/*
-			 * Start a server process. When DEBUG is defined, just run
-			 * in the foreground.
-			 */
+	if (run_as_server) {
+		/*
+		 * Start by binding to the port, the child inherits this socket.
+		 */
+		sock = bind_to_port(bind_ip, bind_port);
+		
+		/*
+		 * Start a server process. When DEBUG is defined, just run
+		 * in the foreground.
+		 */
 #if DEBUG
-			switch (0)
+		switch (0)
 #else
-				switch (fork())
+			switch (fork())
 #endif
-					{
-						case -1:
-							perror("fork()");
-							exit(1);
-
-						case 0:
-							/*
-							 * Ignore some signals.
-							 */
-							signal(SIGHUP, SIG_IGN);
+				{
+				case -1:
+					perror("fork()");
+					exit(1);
+					
+				case 0:
+					/*
+					 * Ignore some signals.
+					 */
+					signal(SIGHUP, SIG_IGN);
 #if DEBUG
-							signal(SIGINT, SIG_IGN);
+					signal(SIGINT, SIG_IGN);
 #endif
-							signal(SIGQUIT, SIG_IGN);
-							signal(SIGTSTP, SIG_IGN);
-							signal(SIGCONT, SIG_IGN);
-							signal(SIGPIPE, alarm_signal);
-							/*signal(SIGALRM, alarm_signal);*/
-
-							/*
-							 * Drop back to an untrusted user.
-							 */
-							setgid(run_gid);
-							initgroups(username, -1);
-							setuid(run_uid);
-
-							/*
-							 * Start a new session and group.
-							 */
-							setsid();
+					signal(SIGQUIT, SIG_IGN);
+					signal(SIGTSTP, SIG_IGN);
+					signal(SIGCONT, SIG_IGN);
+					signal(SIGPIPE, alarm_signal);
+					/*signal(SIGALRM, alarm_signal);*/
+					
+					/*
+					 * Drop back to an untrusted user.
+					 */
+					setgid(run_gid);
+					initgroups(username, -1);
+					setuid(run_uid);
+					
+					/*
+					 * Start a new session and group.
+					 */
+					setsid();
 #ifdef SETPGRP_VOID
-							setpgrp();
+					setpgrp();
 #else
-							setpgrp(0,0);
+					setpgrp(0,0);
 #endif
 #if DEBUG
-							close(2);
-							open("/dev/null", O_WRONLY);
+					close(2);
+					open("/dev/null", O_WRONLY);
 #endif
-							/*
-							 * Handle the server main loop.
-							 */
-							server_main_loop(sock, device_name);
-
-							/*
-							 * Should never exit.
-							 */
-							exit(1);
-					}
-
-			/*
-			 * Parent exits at this stage.
-			 */
-			exit(0);
-		}
+					/*
+					 * Handle the server main loop.
+					 */
+					server_main_loop(sock, device_name, n_dev);
+					
+					/*
+					 * Should never exit.
+					 */
+					exit(1);
+				}
+		
+		/*
+		 * Parent exits at this stage.
+		 */
+		exit(0);
+	}
 
 	signal(SIGHUP, alarm_signal);
 #if DEBUG == 0
@@ -989,11 +960,11 @@ int main (int argc, char** argv)
 #endif
 	/* Starting from inetd */
 
-	serve_client(sockfd, device_name);
+	serve_client(sockfd, device_name, n_dev);
 	return 0;
 }
 
-void send_reply(int rval, io_buffer sock, int len) {
+static void send_reply(int rval, io_buffer sock, int len) {
 	Packet reply = newPacket();
 
 	make_new(reply, 8);
@@ -1007,12 +978,14 @@ void send_reply(int rval, io_buffer sock, int len) {
 	destroyPacket(reply);
 }
 
-void cleanup(int x) {
+static void cleanup(int x) {
 	unlink(XauFileName());
 	exit(-1);
 }
 
-void serve_client(int sockhandle, char* device_name) {
+#include "lockdev.h"
+
+void serve_client(int sockhandle, char **device_name, int n_dev) {
 	Packet opcode;
 	Packet parm;
 
@@ -1066,45 +1039,71 @@ void serve_client(int sockhandle, char* device_name) {
 	if(version == FLOPPYD_PROTOCOL_VERSION_OLD) {
 				/* old protocol */
 		readOnly = 0;
-		devFd = open(device_name, O_RDWR);
+		devFd = open(device_name[0], O_RDWR);
 		
 		if (devFd < 0) {
 			readOnly = 1;
-			devFd = open(device_name, 
+			devFd = open(device_name[0], 
 				     O_RDONLY);
 		}
 		if(devFd < 0) {
 			send_reply(0, sock, devFd);
 			stopLoop = 1;
 		}
+		lock_dev(devFd, !readOnly, NULL);
 	}
 
 
 	while(!stopLoop) {
-		int rval = 0;
+		int dev_nr = 0;
 		/*
 		 * Allow 60 seconds for any activity.
 		 */
 		/*alarm(60);*/
 
 		if (!recv_packet(opcode, sock, 1)) {
-		    fprintf(stderr, "No Recv packet\n");
 			break;
 		}
 /*		if(opcode->data[0] != OP_CLOSE)*/
 		    recv_packet(parm, sock, MAX_DATA_REQUEST);
 
+
 		switch(opcode->data[0]) {
 			case OP_OPRO:
-				devFd = open(device_name, O_RDONLY);
+				if(get_length(parm) >= 4)
+					dev_nr = get_dword(parm,0);
+				else
+					dev_nr = 0;
+				if(dev_nr >= n_dev) {
+					send_reply(0, sock, -1);
+					break;
+				}
+
+				devFd = open(device_name[dev_nr], O_RDONLY);
 #if DEBUG
 				fprintf(stderr, "Device opened\n");
 #endif
+				if(devFd >= 0 && lock_dev(devFd, 0, NULL)) {
+					send_reply(0, sock, -1);
+					break;
+				}
 				send_reply(0, sock, devFd);
 				readOnly = 1;
 				break;
 			case OP_OPRW:
-				devFd = open(device_name, O_RDWR);
+				if(get_length(parm) >= 4)
+					dev_nr = get_dword(parm,0);
+				else
+					dev_nr = 0;
+				if(dev_nr >= n_dev) {
+					send_reply(0, sock, -1);
+					break;
+				}
+				devFd = open(device_name[dev_nr], O_RDWR);
+				if(devFd >= 0 && lock_dev(devFd, 1, NULL)) {
+					send_reply(0, sock, -1);
+					break;
+				}
 				send_reply(0, sock, devFd);
 				readOnly = 0;
 				break;
@@ -1114,8 +1113,8 @@ void serve_client(int sockhandle, char* device_name) {
 #endif
 				read_packet(parm, devFd, get_dword(parm, 0));
 				send_reply(devFd, sock, get_length(parm));
-				send_packet(parm, sock);
-				
+				if(get_length(parm) >= 0)
+					send_packet(parm, sock);
 				break;
 			case OP_WRITE:
 #if DEBUG
