@@ -25,7 +25,7 @@
 typedef unsigned char Byte;
 typedef unsigned long Dword;
 
-char* AuthErrors[] = {
+const char* AuthErrors[] = {
 	"Auth success!",
 	"Auth failed: Packet oversized!",
 	"Auth failed: X-Cookie doesn't match!",
@@ -45,63 +45,21 @@ typedef struct RemoteFile_t {
 	mt_off_t size;
 	int version;
 	int capabilities;
+	int drive;
 } RemoteFile_t;
 
 
-#ifndef HAVE_HTONS
-unsigned short myhtons(unsigned short parm) 
-{
-	Byte val[2];
-	
-	val[0] = (parm >> 8) & 0xff;
-	val[1] = parm        & 0xff;
-
-	return *((unsigned short*) (val));
-}
-#endif
-
-Dword byte2dword(Byte* val) 
-{
-	Dword l;
-	l = (val[0] << 24) + (val[1] << 16) + (val[2] << 8) + val[3];
-
-	return l;
-}	
-
-void dword2byte(Dword parm, Byte* rval) 
-{
-	rval[0] = (parm >> 24) & 0xff;
-	rval[1] = (parm >> 16) & 0xff;
-	rval[2] = (parm >> 8)  & 0xff;
-	rval[3] = parm         & 0xff;
-}
-
-Dword read_dword(int handle) 
-{
-	Byte val[4];
-	
-	read(handle, val, 4);
-
-	return byte2dword(val);
-}
-
-void write_dword(int handle, Dword parm) 
-{
-	Byte val[4];
-
-	dword2byte(parm, val);
-
-	write(handle, val, 4);
-}
+#include "byte_dword.h"
+#include "read_dword.h"
 
 
 /* ######################################################################## */
 
-int authenticate_to_floppyd(RemoteFile_t *floppyd, int sock, char *display)
+static int authenticate_to_floppyd(RemoteFile_t *floppyd, int sock, char *display)
 {
 	off_t filelen;
 	Byte buf[16];
-	char *command[] = { "xauth", "xauth", "extract", "-", 0, 0 };
+	const char *command[] = { "xauth", "xauth", "extract", "-", 0, 0 };
 	char *xcookie;
 	Dword errcode;
 	int l;
@@ -264,9 +222,10 @@ static int floppyd_open(RemoteFile_t *This, int mode)
 		buf[4] = OP_OPRO;
 	else
 		buf[4] = OP_OPRW;
-	dword2byte(1, buf+5);		
+	dword2byte(4, buf+5);
+	dword2byte(This->drive, buf+9);
 
-	write(This->fd, buf, 10);
+	write(This->fd, buf, 13);
        
 	if (read_dword(This->fd) != 8) {
 		errno = EIO;
@@ -442,8 +401,8 @@ static Class_t FloppydFileClass = {
 
 /* ######################################################################## */
 
-int get_host_and_port(const char* name, char** hostname, char **display,
-					  short* port)
+static int get_host_and_port_and_drive(const char* name, char** hostname,
+				       char **display, short* port, int *drive)
 {
 	char* newname = strdup(name);
 	char* p;
@@ -455,10 +414,14 @@ int get_host_and_port(const char* name, char** hostname, char **display,
 	if (*p) p++;
 	*p2 = 0;
 	
-	*port = atoi(p);
-	if (*port == 0) {
-		*port = FLOPPYD_DEFAULT_PORT;	
-	}
+	*port = FLOPPYD_DEFAULT_PORT;	
+	if(*p >= '0' && *p <= '9')
+	  *port = strtoul(p, &p, 0);
+	if(*p == '/')
+	  p++;
+	*drive = 0;
+	if(*p >= '0' && *p <= '9')
+	  *drive = strtoul(p, &p, 0);
 
 	*display = strdup(newname);
 
@@ -498,7 +461,7 @@ static IPaddr_t getipaddress(char *ipaddr)
 		endhostent();
 	}
 	
-#if DEBUG
+#ifdef DEBUG
 	fprintf(stderr, "IP lookup %s -> 0x%08lx\n", ipaddr, ip);
 #endif
 	  
@@ -599,7 +562,8 @@ static int ConnectToFloppyd(RemoteFile_t *floppyd, const char* name,
 	char* hostname;
 	char* display;
 	short port;
-	int rval = get_host_and_port(name, &hostname, &display, &port);
+	int rval = get_host_and_port_and_drive(name, &hostname, &display, 
+					       &port, &floppyd->drive);
 	int sock;
 	int reply;
 	
