@@ -12,7 +12,7 @@
 
 void mbadblocks(int argc, char **argv, int type)
 {
-	int i;
+	unsigned int i;
 	char *in_buf;
 	int in_len;
 	off_t start;
@@ -30,7 +30,7 @@ void mbadblocks(int argc, char **argv, int type)
 
 	init_mp(&mp);
 
-	Dir = open_root_dir(argv[1][0], O_RDWR);
+	Dir = open_root_dir(argv[1][0], O_RDWR, NULL);
 	if (!Dir) {
 		fprintf(stderr,"%s: Cannot initialize drive\n", argv[0]);
 		exit(1);
@@ -45,13 +45,13 @@ void mbadblocks(int argc, char **argv, int type)
 		exit(1);
 	}
 	for(i=0; i < Fs->clus_start; i++ ){
-		ret = READS(Fs->Next, 
-					in_buf, sectorsToBytes((Stream_t*)Fs, i), Fs->sector_size);
+		ret = READS(Fs->Next, in_buf, 
+			    sectorsToBytes((Stream_t*)Fs, i), Fs->sector_size);
 		if( ret < 0 ){
 			perror("early error");
 			exit(1);
 		}
-		if(ret < Fs->sector_size){
+		if(ret < (signed int) Fs->sector_size){
 			fprintf(stderr,"end of file in file_read\n");
 			exit(1);
 		}
