@@ -13,7 +13,7 @@
 #include "floppyd_io.h"
 #include "plain_io.h"
 
-void usage() 
+static void usage(void) 
 {
 	fprintf(stderr, "Mtools version %s, dated %s\n", 
 		mversion, mdate);
@@ -22,7 +22,11 @@ void usage()
 	exit(1);
 }
 
+#ifdef __CYGWIN__
+#define BUF_SIZE 512
+#else
 #define BUF_SIZE 16000
+#endif
 
 static size_t bufLen(size_t blocksize, mt_size_t totalSize, mt_off_t address)
 {
