@@ -37,8 +37,6 @@
 #endif
 
 
-extern int errno;
-
 static int init_geometry_boot(struct bootsector *boot, struct device *dev,
 			       int sectors0, int rate_0, int rate_any,
 			       unsigned long *tot_sectors, int keepBoot)
@@ -127,7 +125,7 @@ static int init_geometry_boot(struct bootsector *boot, struct device *dev,
 			boot->jump[0] = 0xeb;
 			boot->jump[1] = 0;
 			boot->jump[2] = 0x90;
-			strncpy(boot->banner, "MTOOL399", 8);
+			strncpy(boot->banner, "MTOO3911", 8);
 			/* It looks like some versions of DOS are
 			 * rather picky about this, and assume default
 			 * parameters without this, ignoring any
@@ -169,7 +167,7 @@ static int comp_fat_bits(Fs_t *Fs, int estimate,
 	}
 
 	if(!estimate) {
-		int min_fat16_size;
+		unsigned int min_fat16_size;
 
 		if(needed_fat_bits > 12)
 			return needed_fat_bits;
@@ -343,10 +341,10 @@ static void calc_fat_size(Fs_t *Fs, unsigned long tot_sectors)
 	unsigned long numerator;
 	unsigned long denominator;
 	int fat_nybbles;
-	int slack;
+	unsigned int slack;
 	int printGrowMsg=1; /* Should we print "growing FAT" messages ?*/
 	
-#if DEBUG
+#ifdef DEBUG
 	fprintf(stderr, "Fat start=%d\n", Fs->fat_start);
 	fprintf(stderr, "tot_sectors=%lu\n", tot_sectors);
 	fprintf(stderr, "dir_len=%d\n", Fs->dir_len);
@@ -361,7 +359,7 @@ static void calc_fat_size(Fs_t *Fs, unsigned long tot_sectors)
 	   Fs->cluster_size %2 == 0)
 		rem_sect--;
 
-#if DEBUG
+#ifdef DEBUG
 	fprintf(stderr, "Rem sect=%lu\n", rem_sect);
 #endif
 
@@ -389,7 +387,7 @@ static void calc_fat_size(Fs_t *Fs, unsigned long tot_sectors)
 		 * rather than multiplying the numerator */
 		denominator = denominator / fat_nybbles;
 
-#if DEBUG
+#ifdef DEBUG
 	fprintf(stderr, "Numerator=%lu denominator=%lu\n",
 		numerator, denominator);
 #endif
@@ -413,7 +411,7 @@ static void calc_fat_size(Fs_t *Fs, unsigned long tot_sectors)
 				fat_nybbles - 2);
 		fprintf(stderr, " to %d\n", Fs->num_clus);
 	}
-#if DEBUG
+#ifdef DEBUG
 	fprintf(stderr, "Num_clus=%d fat_len=%d nybbles=%d\n",
 		Fs->num_clus, Fs->fat_len, fat_nybbles);
 #endif
@@ -430,6 +428,10 @@ static void calc_fat_size(Fs_t *Fs, unsigned long tot_sectors)
 	 * of sectors. This means that if we reduced the number of clusters
 	 * above, we will have to grow the FAT in order to take up any excess
 	 * sectors... */
+#ifdef HAVE_ASSERT_H
+	assert(rem_sect >= Fs->num_clus * Fs->cluster_size + 
+	       Fs->fat_len * Fs->num_fat);
+#endif
 	slack = rem_sect - 
 		Fs->num_clus * Fs->cluster_size - 
 		Fs->fat_len * Fs->num_fat;
@@ -557,9 +559,9 @@ struct OldDos_t old_dos[]={
 {    1,  8,  1, 1, 1, 1, 0xf0 },
 };
 
-static int old_dos_size_to_geom(int size, int *cyls, int *heads, int *sects)
+static int old_dos_size_to_geom(size_t size, int *cyls, int *heads, int *sects)
 {
-	int i;
+	unsigned int i;
 	size = size * 2;
 	for(i=0; i < sizeof(old_dos) / sizeof(old_dos[0]); i++){
 		if (old_dos[i].sectors * 
@@ -578,7 +580,7 @@ static int old_dos_size_to_geom(int size, int *cyls, int *heads, int *sects)
 static void calc_fs_parameters(struct device *dev, unsigned long tot_sectors,
 			       struct Fs_t *Fs, struct bootsector *boot)
 {
-	int i;
+	unsigned int i;
 
 	for(i=0; i < sizeof(old_dos) / sizeof(old_dos[0]); i++){
 		if (dev->sectors == old_dos[i].sectors &&
@@ -685,6 +687,53 @@ static void usage(void)
 	exit(1);
 }
 
+#ifdef OS_linux
+static int get_block_geom(int fd, struct MT_STAT *buf, struct device *dev,
+			  char *errmsg) {
+	struct hd_geometry geom;
+	long size;
+	int heads=dev->heads;
+	int sectors=dev->sectors;
+	int sect_per_track;
+
+	if (ioctl(fd, HDIO_GETGEO, &geom) < 0) {
+		sprintf(errmsg, "Could not get geometry of device (%s)",
+			strerror(errno));
+		return -1;
+	}
+	
+	if (ioctl(fd, BLKGETSIZE, &size) < 0) {
+		sprintf(errmsg, "Could not get size of device (%s)",
+			strerror(errno));
+		return -1;
+	}
+	
+	if(!heads)
+		heads = geom.heads;
+	if(!sectors)
+		sectors = geom.sectors;
+
+	sect_per_track = heads * sectors;
+	if(!dev->hidden) {
+		int hidden;
+		hidden = geom.start % sect_per_track;
+		if(hidden && hidden != sectors) {
+			sprintf(errmsg,
+				"Hidden (%d) does not match sectors (%d)\n",
+				hidden, sectors);
+			return -1;
+		}
+		dev->hidden = hidden;
+	}
+	dev->heads = heads;
+	dev->sectors = sectors;
+	if(!dev->tracks)
+		dev->tracks = (size + dev->hidden) / sect_per_track;
+	size = dev->tracks * dev->heads * dev->sectors + dev->hidden;
+	return 0;
+}
+#endif
+
 void mformat(int argc, char **argv, int dummy)
 {
 	int r; /* generic return value */
@@ -702,7 +751,7 @@ void mformat(int argc, char **argv, int dummy)
 	int bootOffset;
 
 #ifdef USE_XDF
-	int i;
+	unsigned int i;
 	int format_xdf = 0;
 	struct xdf_info info;
 #endif
@@ -724,6 +773,7 @@ void mformat(int argc, char **argv, int dummy)
 	unsigned long serial;
  	int serial_set;
 	int fsVersion;
+	int mediaDesc=-1;
 
 	mt_off_t maxSize;
 
@@ -743,7 +793,18 @@ void mformat(int argc, char **argv, int dummy)
 	Fs.cluster_size = 0;
 	Fs.refs = 1;
 	Fs.dir_len = 0;
+	if(getenv("MTOOLS_DIR_LEN")) {
+	  Fs.dir_len = atoi(getenv("MTOOLS_DIR_LEN"));
+	  if(Fs.dir_len <= 0)
+	    Fs.dir_len=0;
+	}
 	Fs.fat_len = 0;
+	Fs.num_fat = 2;
+	if(getenv("MTOOLS_NFATS")) {
+	  Fs.num_fat = atoi(getenv("MTOOLS_NFATS"));
+	  if(Fs.num_fat <= 0)
+	    Fs.num_fat=2;
+	}
 	Fs.Class = &FsClass;	
 	rate_0 = mtools_rate_0;
 	rate_any = mtools_rate_any;
@@ -751,7 +812,7 @@ void mformat(int argc, char **argv, int dummy)
 	/* get command line options */
 	while ((c = getopt(argc,argv,
 			   "i:148f:t:n:v:qub"
-			   "kB:r:L:IFCc:Xh:s:l:N:H:M:S:2:30:Aa"))!= EOF) {
+			   "kB:r:L:IFCc:Xh:s:l:N:H:M:S:2:30:Aad:m:"))!= EOF) {
 		switch (c) {
 			case 'i':
 				set_cmd_line_image(optarg, 0);
@@ -815,6 +876,10 @@ void mformat(int argc, char **argv, int dummy)
 				argssize = atoi(optarg) | 0x80;
 				if(argssize < 0x81)
 					usage();
+				if(argssize >= 0x87) {
+					fprintf(stderr, "argssize must be less than 6\n");
+					usage();
+				}
 				break;
 
 #ifdef USE_XDF
@@ -858,7 +923,7 @@ void mformat(int argc, char **argv, int dummy)
 				break;
 
 			case 'C':
-				create = O_CREAT;
+				create = O_CREAT | O_TRUNC;
 				break;
 
 			case 'H':
@@ -891,7 +956,12 @@ void mformat(int argc, char **argv, int dummy)
 			case 'h':
 				argheads = atoi(optarg);
 				break;
-
+			case 'd':
+				Fs.num_fat = atoi(optarg);
+				break;
+			case 'm':
+				mediaDesc = strtoul(optarg,0,0);
+				break;
 			default:
 				usage();
 		}
@@ -969,36 +1039,16 @@ void mformat(int argc, char **argv, int dummy)
 		if ((!used_dev.tracks || !used_dev.heads || !used_dev.sectors) &&
 			(!IS_SCSI(dev))) {
 			int fd= get_fd(Fs.Direct);
-			struct MT_STAT buf;
+			struct MT_STAT stbuf;
 
-			if (MT_FSTAT(fd, &buf) < 0) {
+			if (MT_FSTAT(fd, &stbuf) < 0) {
 				sprintf(errmsg, "Could not stat file (%s)", strerror(errno));
 				continue;						
 			}
 
-			if (S_ISBLK(buf.st_mode)) {
-				struct hd_geometry geom;
-				long size;
-				int sect_per_track;
-
-				if (ioctl(fd, HDIO_GETGEO, &geom) < 0) {
-					sprintf(errmsg, "Could not get geometry of device (%s)",
-							strerror(errno));
-					continue;
-				}
-
-				if (ioctl(fd, BLKGETSIZE, &size) < 0) {
-					sprintf(errmsg, "Could not get size of device (%s)",
-							strerror(errno));
-					continue;
-				}
-
-				sect_per_track = geom.heads * geom.sectors;
-				used_dev.heads = geom.heads;
-				used_dev.sectors = geom.sectors;
-				used_dev.hidden = geom.start % sect_per_track;
-				used_dev.tracks = (size + used_dev.hidden) / sect_per_track;
-			}
+			if (S_ISBLK(stbuf.st_mode) &&
+			    get_block_geom(fd, &stbuf, &used_dev, errmsg) < 0)
+				continue;
 		}
 #endif
 
@@ -1027,10 +1077,10 @@ void mformat(int argc, char **argv, int dummy)
 
 		SET_INT(Fs.sector_size, msize);
 		{
-		    int i;
-		    for(i = 0; i < 31; i++) {
-			if (Fs.sector_size == 1 << i) {
-			    Fs.sectorShift = i;
+		    unsigned int j;
+		    for(j = 0; j < 31; j++) {
+			if (Fs.sector_size == (unsigned int) (1 << j)) {
+			    Fs.sectorShift = j;
 			    break;
 			}
 		    }
@@ -1048,7 +1098,7 @@ void mformat(int argc, char **argv, int dummy)
 		/* do a "test" read */
 		if (!create &&
 		    READS(Fs.Direct, (char *) buf, 0, Fs.sector_size) != 
-		    Fs.sector_size) {
+		    (signed int) Fs.sector_size) {
 			sprintf(errmsg, 
 				"Error reading from '%s', wrong parameters?",
 				name);
@@ -1065,12 +1115,22 @@ void mformat(int argc, char **argv, int dummy)
 		exit(1);
 	}
 
+	/* calculate the total number of sectors */
+	tot_sectors = used_dev.tracks*used_dev.heads*used_dev.sectors - used_dev.hidden;
+
+	/* create the image file if needed */
+	if (create) {
+		WRITES(Fs.Direct, (char *) buf,
+		       sectorsToBytes((Stream_t*)&Fs, tot_sectors-1),
+		       Fs.sector_size);
+	}
+
 	/* the boot sector */
 	boot = (struct bootsector *) buf;
 	if(bootSector) {
 		int fd;
 
-		fd = open(bootSector, O_RDONLY | O_LARGEFILE);
+		fd = open(bootSector, O_RDONLY | O_BINARY | O_LARGEFILE);
 		if(fd < 0) {
 			perror("open boot sector");
 			exit(1);
@@ -1097,14 +1157,11 @@ void mformat(int argc, char **argv, int dummy)
 			   blocksize);
 	Fs.Buffer = 0;
 
-	boot->nfat = Fs.num_fat = 2;
+	boot->nfat = Fs.num_fat;
 	if(!keepBoot)
 		set_word(boot->jump + 510, 0xaa55);
 	
-	/* get the parameters */
-	tot_sectors = used_dev.tracks * used_dev.heads * used_dev.sectors - 
-		DWORD(nhs);
-
+	/* Initialize the remaining parameters */
 	set_word(boot->nsect, used_dev.sectors);
 	set_word(boot->nheads, used_dev.heads);
 
@@ -1183,12 +1240,6 @@ void mformat(int argc, char **argv, int dummy)
 		boot->banner[7] = random();
 	}		
 
-	if (create) {
-		WRITES(Fs.Direct, (char *) buf,
-		       sectorsToBytes((Stream_t*)&Fs, tot_sectors-1),
-		       Fs.sector_size);
-	}
-
 	if(!keepBoot)
 		inst_boot_prg(boot, bootOffset);
 	/* Mimic 3.8 behavior, else 2m disk do not work (???)
@@ -1201,6 +1252,8 @@ void mformat(int argc, char **argv, int dummy)
 	}
 	if(used_dev.use_2m & 0x7f)
 		Fs.num_fat = 1;
+	if(mediaDesc != -1)
+		boot->descr=mediaDesc;
 	Fs.lastFatSectorNr = 0;
 	Fs.lastFatSectorData = 0;
 	zero_fat(&Fs, boot->descr);
@@ -1217,7 +1270,7 @@ void mformat(int argc, char **argv, int dummy)
 
 	format_root(&Fs, label, boot);
 	WRITES((Stream_t *)&Fs, (char *) boot, (mt_off_t) 0, Fs.sector_size);
-	if(Fs.fat_bits == 32 && WORD(ext.fat32.backupBoot) != MAX32) {
+	if(Fs.fat_bits == 32 && WORD(ext.fat32.backupBoot) != MAX16) {
 		WRITES((Stream_t *)&Fs, (char *) boot, 
 		       sectorsToBytes((Stream_t*)&Fs, WORD(ext.fat32.backupBoot)),
 		       Fs.sector_size);
