@@ -56,7 +56,7 @@ static void set_offset(hsc *h, int offset, int heads, int sectors)
 }
 
 void setBeginEnd(struct partition *partTable, int begin, int end,
-				 int heads, int sectors, int activate, int type)
+		 int heads, int sectors, int activate, int type)
 {
 	set_offset(&partTable->start, begin, heads, sectors);
 	set_offset(&partTable->end, end-1, heads, sectors);
@@ -78,11 +78,12 @@ void setBeginEnd(struct partition *partTable, int begin, int end,
 }
 
 int consistencyCheck(struct partition *partTable, int doprint, int verbose,
-		     int *has_activated, int *last_end, int *j, 
+		     int *has_activated, unsigned int *last_end,
+		     unsigned int *j, 
 		     struct device *used_dev, int target_partition)
 {
-	int i;
-	int inconsistency;
+	unsigned int i;
+	unsigned int inconsistency;
 	
 	*j = 0;
 	*last_end = 1;
@@ -105,7 +106,8 @@ int consistencyCheck(struct partition *partTable, int doprint, int verbose,
 			inconsistency=1;
 		}
 		
-		if(*j && *last_end > BEGIN(partTable[i])) {
+		if(*j && 
+		   *last_end > BEGIN(partTable[i])) {
 			fprintf(stderr,
 				"Partitions %d and %d badly ordered or overlapping\n",
 				*j,i);
@@ -249,7 +251,7 @@ static void usage(void)
 	fprintf(stderr, 
 		"Mtools version %s, dated %s\n", mversion, mdate);
 	fprintf(stderr, 
-		"Usage: %s [-pradcv] [-I [-B bootsect-template] [-s sectors] "
+		"Usage: %s [-pradcv] [-I] [-B bootsect-template] [-s sectors] "
 			"[-t cylinders] "
 		"[-h heads] [-T type] [-b begin] [-l length] "
 		"drive\n", progname);
@@ -261,7 +263,7 @@ void mpartition(int argc, char **argv, int dummy)
 	Stream_t *Stream;
 	unsigned int dummy2;
 
-	int i,j;
+	unsigned int i,j;
 
 	int sec_per_cyl;
 	int doprint = 0;
@@ -269,14 +271,14 @@ void mpartition(int argc, char **argv, int dummy)
 	int create = 0;
 	int force = 0;
 	int length = 0;
-	int remove = 0;
+	int do_remove = 0;
 	int initialize = 0;
-	int tot_sectors=0;
+	unsigned int tot_sectors=0;
 	int type = 0;
 	int begin_set = 0;
 	int size_set = 0;
 	int end_set = 0;
-	int last_end = 0;
+	unsigned int last_end = 0;
 	int activate = 0;
 	int has_activated = 0;
 	int inconsistency=0;
@@ -326,7 +328,7 @@ void mpartition(int argc, char **argv, int dummy)
 				doprint = 1;
 				break;
 			case 'r':
-				remove = 1;
+				do_remove = 1;
 				dirty = 1;
 				break;
 			case 'I':
@@ -503,7 +505,7 @@ void mpartition(int argc, char **argv, int dummy)
 	if(initialize) {
 		if (bootSector) {
 			int fd;
-			fd = open(bootSector, O_RDONLY | O_LARGEFILE);
+			fd = open(bootSector, O_RDONLY | O_BINARY | O_LARGEFILE);
 			if (fd < 0) {
 				perror("open boot sector");
 				exit(1);
@@ -522,7 +524,7 @@ void mpartition(int argc, char **argv, int dummy)
 		inconsistency = 1;
 	}
 	
-	if(remove){
+	if(do_remove){
 		if(!partTable[dev->partition].sys_ind)
 			fprintf(stderr,
 				"Partition for drive %c: does not exist\n",
