@@ -8,15 +8,17 @@
 #define MAX_SECTOR	8192   		/* largest sector size */
 #define MDIR_SIZE	32		/* MSDOS directory entry size in bytes*/
 #define MAX_CLUSTER	8192		/* largest cluster size */
+#ifndef MAX_PATH
 #define MAX_PATH	128		/* largest MSDOS path length */
+#endif
 #define MAX_DIR_SECS	64		/* largest directory (in sectors) */
 #define MSECTOR_SIZE    msector_size
 
 #define NEW		1
 #define OLD		0
 
-#define _WORD(x) ((unsigned char)(x)[0] + (((unsigned char)(x)[1]) << 8))
-#define _DWORD(x) (_WORD(x) + (_WORD((x)+2) << 16))
+#define _WORD(x) ((unsigned short)((unsigned char)(x)[0] + (((unsigned char)(x)[1]) << 8)))
+#define _DWORD(x) ((unsigned int)(_WORD(x) + (_WORD((x)+2) << 16)))
 
 #define DELMARK ((char) 0xe5)
 
@@ -39,6 +41,7 @@ struct directory {
 #define EXTCASE 0x10
 #define BASECASE 0x8
 
+#define MAX16 0xffff
 #define MAX32 0xffffffff
 #define MAX_SIZE 0x7fffffff
 
@@ -111,7 +114,7 @@ typedef struct label_blk_t {
 } label_blk_t;
 
 /* FAT32 specific info in the bootsector */
-typedef struct fat32_t {
+struct fat32_t {
 	unsigned char bigFat[4];	/* 36 nb of sectors per FAT */
 	unsigned char extFlags[2];     	/* 40 extension flags */
 	unsigned char fsVersion[2];	/* 42 ? */
@@ -121,7 +124,7 @@ typedef struct fat32_t {
 	unsigned char reserved[6];	/* 52 ? */
 	unsigned char reserved2[6];	/* 52 ? */
 	struct label_blk_t labelBlock;
-} fat32; /* ends at 58 */
+}; /* ends at 58 */
 
 typedef struct oldboot_t {
 	struct label_blk_t labelBlock;
@@ -161,6 +164,8 @@ struct bootsector {
 		struct oldboot_t old;
 	} ext;
 };
+#define MAX_BOOT 4096
+
 
 #define CHAR(x) (boot->x[0])
 #define WORD(x) (_WORD(boot->x))
@@ -169,13 +174,13 @@ struct bootsector {
 
 
 extern struct OldDos_t {
-	int tracks;
-	int sectors;
-	int heads;
+	unsigned int tracks;
+	unsigned int sectors;
+	unsigned int heads;
 	
-	int dir_len;
-	int cluster_size;
-	int fat_len;
+	unsigned int dir_len;
+	unsigned int cluster_size;
+	unsigned int fat_len;
 
 	int media;
 } old_dos[];
