@@ -17,6 +17,7 @@ extern int lockf(int, int, off_t);  /* SCO has no proper include file for lockf
 #define VOLD_FLAG 32
 #define FLOPPYD_FLAG 64
 #define FILTER_FLAG 128
+#define SWAP_FLAG 256 
 
 #define IS_SCSI(x)  ((x) && ((x)->misc_flags & SCSI_FLAG))
 #define IS_PRIVILEGED(x) ((x) && ((x)->misc_flags & PRIV_FLAG))
@@ -24,6 +25,7 @@ extern int lockf(int, int, off_t);  /* SCO has no proper include file for lockf
 #define IS_MFORMAT_ONLY(x) ((x) && ((x)->misc_flags & MFORMAT_ONLY_FLAG))
 #define SHOULD_USE_VOLD(x) ((x)&& ((x)->misc_flags & VOLD_FLAG))
 #define SHOULD_USE_XDF(x) ((x)&& ((x)->misc_flags & USE_XDF_FLAG))
+#define DO_SWAP(x)  ((x) && ((x)->misc_flags & SWAP_FLAG))
 
 typedef struct device {
 	const char *name;       /* full path to device */
@@ -53,7 +55,7 @@ typedef struct device {
 
 	/* internal variables */
 	int file_nr;		/* used during parsing */
-	int blocksize;	        /* size of disk block in bytes */
+	unsigned int blocksize;	  /* size of disk block in bytes */
 
 	const char *cfg_filename; /* used for debugging purposes */
 } device_t;
@@ -130,7 +132,7 @@ void setup_signal(void);
 if(source)target=source
 
 
-UNUSED(static inline int compare (long ref, long testee))
+UNUSED(static __inline__ int compare (long ref, long testee))
 {
 	return (ref && ref != testee);
 }
@@ -148,18 +150,20 @@ extern unsigned int mtools_no_vfat;
 extern unsigned int mtools_numeric_tail;
 extern unsigned int mtools_dotted_dir;
 extern unsigned int mtools_twenty_four_hour_clock;
-extern char *mtools_date_string;
+extern const char *mtools_date_string;
 extern unsigned int mtools_rate_0, mtools_rate_any;
 extern int mtools_raw_tty;
 
 extern int batchmode;
 
+char get_default_drive(void);
+void set_cmd_line_image(char *img, int flags);
 void read_config(void);
 extern struct device *devices;
 extern struct device const_devices[];
 extern const int nr_const_devices;
 
-#define New(type) ((type*)(malloc(sizeof(type))))
+#define New(type) ((type*)(calloc(1,sizeof(type))))
 #define Grow(adr,n,type) ((type*)(realloc((char *)adr,n*sizeof(type))))
 #define Free(adr) (free((char *)adr));
 #define NewArray(size,type) ((type*)(calloc((size),sizeof(type))))
@@ -168,6 +172,7 @@ void mattrib(int argc, char **argv, int type);
 void mbadblocks(int argc, char **argv, int type);
 void mcat(int argc, char **argv, int type);
 void mcd(int argc, char **argv, int type);
+void mclasserase(int argc, char **argv, int type);
 void mcopy(int argc, char **argv, int type);
 void mdel(int argc, char **argv, int type);
 void mdir(int argc, char **argv, int type);
@@ -196,7 +201,7 @@ extern const char *progname;
 
 void precmd(struct device *dev);
 
-void print_sector(char *message, unsigned char *data, int size);
+void print_sector(const char *message, unsigned char *data, int size);
 time_t getTimeNow(time_t *now);
 
 #ifdef USING_NEW_VOLD
@@ -220,13 +225,18 @@ int ask_confirmation(const char *, const char *, const char *);
 char *get_homedir(void);
 #define EXPAND_BUF 2048
 const char *expand(const char *, char *);
-const char *fix_mcwd(char *);
 FILE *open_mcwd(const char *mode);
 void unlink_mcwd(void);
 
-int safePopenOut(char **command, char *output, int len);
+#ifndef OS_mingw32msvc
+int safePopenOut(const char **command, char *output, int len);
+#endif
 
 #define ROUND_DOWN(value, grain) ((value) - (value) % (grain))
 #define ROUND_UP(value, grain) ROUND_DOWN((value) + (grain)-1, (grain))
 
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+
 #endif
