@@ -52,7 +52,7 @@ static int zip_cmd(int priv, int fd, unsigned char cdb[6], int clen,
 	return r;
 }
 
-int test_mounted ( char *dev )
+static int test_mounted ( char *dev )
 {
 #ifdef HAVE_MNTENT_H
 	struct mntent	*mnt;
@@ -72,10 +72,14 @@ int test_mounted ( char *dev )
 	if (!S_ISBLK (st_dev.st_mode)) /* not a block device, cannot 
 					* be mounted */
 		return 0;
-	
-	if ((mtab = setmntent (MOUNTED, "r")) == NULL) {
+
+#ifndef _PATH_MOUNTED
+# define _PATH_MOUNTED "/etc/mtab"
+#endif
+
+	if ((mtab = setmntent (_PATH_MOUNTED, "r")) == NULL) {
 		fprintf (stderr, "%s: can't open %s.\n",
-			 progname, MOUNTED);
+			 progname, _PATH_MOUNTED);
 		exit(1);
 	}
 	
@@ -225,8 +229,11 @@ void mzip(int argc, char **argv, int type)
 	break;
 	
 	/* get command line options */
-	while ((c = getopt(argc, argv, "efpqrwxu")) != EOF) {
+	while ((c = getopt(argc, argv, "i:efpqrwxu")) != EOF) {
 		switch (c) {
+			case 'i':
+				set_cmd_line_image(optarg, SCSI_FLAG);
+				break;
 			case 'f':
 				if (get_real_uid()) {
 					fprintf(stderr, 
@@ -264,7 +271,7 @@ void mzip(int argc, char **argv, int type)
 	     (!argv[optind][0] || argv[optind][1] != ':')))
 		usage();
 	
-	drive = toupper(argc - optind == 1 ? argv[argc - 1][0] : 'a');
+	drive = toupper(argc - optind == 1 ? argv[argc - 1][0] : ':');
 	
 	for (dev = devices; dev->name; dev++) {
 		unsigned char cdb[6] = { 0, 0, 0, 0, 0, 0 };
@@ -297,9 +304,9 @@ void mzip(int argc, char **argv, int type)
 
 		if(IS_PRIVILEGED(dev))
 			reclaim_privs();
-		fd = scsi_open(name, O_RDONLY | 
+		fd = scsi_open(name, O_RDONLY
 #ifdef O_NDELAY
-			       O_NDELAY
+			       | O_NDELAY
 #endif
 			       , 0644,
 			       &extra_data);
@@ -327,7 +334,7 @@ void mzip(int argc, char **argv, int type)
 			continue;
 		}
 		
-#if DEBUG
+#ifdef DEBUG
 		fprintf(stderr, "device: %s\n\tvendor: %.8s\n\tproduct: %.16s\n"
 			"\trevision: %.4s\n", name, inq_data.vendor,
 			inq_data.product, inq_data.revision);
@@ -341,6 +348,8 @@ void mzip(int argc, char **argv, int type)
 				 inq_data.product, sizeof inq_data.product) &&
 		     strncasecmp("ZIP 250         ",
 				 inq_data.product, sizeof inq_data.product) &&
+		     strncasecmp("ZIP 750         ",
+				 inq_data.product, sizeof inq_data.product) &&
 		     strncasecmp("JAZ 1GB         ",
 				 inq_data.product, sizeof inq_data.product) &&
 		     strncasecmp("JAZ 2GB         ",
@@ -396,7 +405,7 @@ void mzip(int argc, char **argv, int type)
 		}
 
 		if ((oldMode & unlockMask) == 1) {  /* unlock first */
-			char *s, *passwd;
+			char *s;
 			passwd = "APlaceForYourStuff";
 			if ((s = strchr(passwd, '\n'))) *s = '\0';  /* chomp */
 			iomega_command(IS_PRIVILEGED(dev), fd, unlockMode, 
@@ -406,7 +415,7 @@ void mzip(int argc, char **argv, int type)
 		if ((get_zip_status(IS_PRIVILEGED(dev), fd, extra_data) & 
 		     unlockMask) == 1) {
 			/* unlock first */
-			char *s, *passwd;
+			char *s;
 			passwd = getpass("Password: ");
 			if ((s = strchr(passwd, '\n'))) *s = '\0';  /* chomp */
 			if((ret=iomega_command(IS_PRIVILEGED(dev), fd, 
@@ -457,7 +466,7 @@ void mzip(int argc, char **argv, int type)
 	}
 	
 	if (request & ZIP_STATUS) {
-		char *unlocked;
+		const char *unlocked;
 
 		if(oldMode & 8)
 			unlocked = " and unlocked until eject";
