@@ -9,6 +9,7 @@
 
 void precmd(struct device *dev)
 {
+#ifndef OS_mingw32msvc
 	int status;
 	pid_t pid;
 
@@ -21,11 +22,12 @@ void precmd(struct device *dev)
 			exit(1);
 			break;
 		case 0: /* the son */
-			execl("/bin/sh", "sh", "-c", dev->precmd, 0);
+			execl("/bin/sh", "sh", "-c", dev->precmd, (char *)NULL);
 			break;
 		default:
 			wait(&status);
 			break;
 	}
+#endif
 }
 		
