@@ -2,6 +2,35 @@
 #include "msdos.h"
 #include "mtools.h"
 
+int noPrivileges=0;
+
+#ifdef OS_mingw32msvc
+void reclaim_privs(void)
+{
+}
+
+void drop_privs(void)
+{
+}
+
+void destroy_privs(void)
+{
+}
+
+uid_t get_real_uid(void)
+{
+	return 0;
+}
+
+void init_privs(void)
+{
+}
+
+void closeExec(int fd)
+{
+}
+
+#else
 /*#define PRIV_DEBUG*/
 
 #if 0
@@ -15,7 +44,7 @@ int setresuid(int a, int b, int c)
 }
 #endif
 
-static inline void print_privs(const char *message)
+static __inline__ void print_privs(const char *message)
 {
 #ifdef PRIV_DEBUG
 	/* for debugging purposes only */
@@ -24,8 +53,6 @@ static inline void print_privs(const char *message)
 #endif
 }
 
-int noPrivileges=0;
-
 
 static gid_t rgid, egid;
 static uid_t ruid, euid;
@@ -47,7 +74,7 @@ static uid_t ruid, euid;
  */
 
 
-static inline void Setuid(uid_t uid)
+static __inline__ void Setuid(uid_t uid)
 {
 #if defined HAVE_SETEUID || defined HAVE_SETRESUID
 	if(euid == 0) {
@@ -164,3 +191,4 @@ void closeExec(int fd)
 	fcntl(fd, F_SETFD, 1);
 #endif
 }
+#endif
