@@ -16,8 +16,6 @@
 #include "devices.h"
 #include "xdf_io.h"
 
-extern int errno;
-
 /* Algorithms can't be patented */
 
 typedef struct sector_map {
@@ -42,15 +40,15 @@ struct {
     {	{0,3},	{0,4},	{1,6},	{0,2},	{1,2},	{0,6},	{1,4},	{1,3},	{0,0} }
   },
   {
-    46, 37, 0x43, 1,
+    46, 37, 1, 0x43,
     {	{0,3},	{0,4},	{0,5},	{0,7},	{1,3},	{1,4},	{1,5},	{1,7},	{0,0} }
   },
   {
-    24, 20, 0, 1,
+    24, 20, 1, 0,
     {	{0,5},	{1,6},	{0,6},	{1, 5} }
   },
   {
-    48, 41, 0, 1,
+    48, 41, 1, 0,
     {	{0,6},	{1,7},	{0,7},	{1, 6} }
   }
 };
@@ -490,9 +488,9 @@ static int xdf_write(Stream_t *Stream, char *buf, mt_off_t where, size_t len)
 	len2 = load_bounds(This, begin, end);
 	if(len2 < 0)
 		return len2;
-	maximize(end, len2);
+	maximize(end, (off_t)len2);
 	len2 -= begin;
-	maximize(len, len2);
+	maximize(len, (off_t)len2);
 	memcpy(This->buffer + begin, buf, len);
 	mark_dirty(This, begin, end);
 	return end - begin;
@@ -584,7 +582,7 @@ Stream_t *XdfOpen(struct device *dev, char *name,
 	Xdf_t *This;
 	off_t begin, end;
 	struct bootsector *boot;
-	int type;
+	unsigned int type;
 
 	if(dev && (!SHOULD_USE_XDF(dev) || check_geom(dev, 0, 0)))
 		return NULL;
@@ -659,6 +657,7 @@ Stream_t *XdfOpen(struct device *dev, char *name,
 			This->map = xdf_table[type].map;
 			This->track0_size = xdf_table[type].track0_size;
 			This->rootskip = xdf_table[type].rootskip;
+			This->rate = xdf_table[type].rate;
 			break;
 		}
 	}
